CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  CANNON1_PIN   = 26
  CANNON2_PIN   = 24
  CANNON3_S_PIN = 27 ' in place of 3, a servo

  ' cannon servo parms =======
  CANNON_S_INITIAL= 1982
  CANNON_S_FINAL  = 1800

  ' attack cts =========

  SHOOT_WAIT_TIME = 1200  
  SHOOT_THRESH    = 12
  LONG_TURN       = 100 'was 200
  SHORT_TURN      = 25
  VERY_SHORT_TURN =  5
  FWD_TIME        = 300
  REV_TIME        = 100

  IRBL_PIN        = 0
  IRL_PIN         = 1
  IRFL_PIN        = 2  
  IRFR_PIN        = 3
  IRR_PIN         = 4
  IRBR_PIN        = 5 

  #1, ATK_NOTHING, ATK_FRONT, ATK_LEFT, ATK_RIGHT, ATK_SHOOT, ATK_PURSUIT
  #1, IR_NOTHING, IR_LEFT, IR_LEFT_FRONT, IR_RIGHT, IR_RIGHT_FRONT, IR_LOCK

  OUT = 1
  BUFSIZE = 25

  SERVO_ENABLED = false

OBJ
  irdet  : "irdetect"
  servos : "Servo32" ' servos contol, 1 cog
  clk    : "clock"
  omot   : "open_motor"
  num    : "simple_numbers"
 
VAR
  long stackATK[120]                         
  byte ir_detect_state
  long ir_detect_state_ptr
  long ir_detect_ctr_ptr
  long ir_detect_ptr
  long beh_ctrl_ptr
  long motorL_ptr
  long motorR_ptr
  long messages_ptr
  long msg_disabled
  long msg_R
  long msg_L
  long msg_rev
  long msg_shoot
  long msg_shoot1
  long msg_shoot2
  long msg_shoot3
  long msg_pursuit
  long msg_front
  long msg_start
  long msg_nothing
 

  long msg_buf[BUFSIZE]

  long bullets_left 
  long attack_accumulator

PUB start(ir_detect_state_ptr_, ir_detect_ctr_ptr_, ir_detect_ptr_, beh_ctrl_ptr_, motorL_ptr_, motorR_ptr_, messages_ptr_) 
  ir_detect_state_ptr := ir_detect_state_ptr_
  ir_detect_ctr_ptr   := ir_detect_ctr_ptr_
  ir_detect_ptr       := ir_detect_ptr_
  beh_ctrl_ptr        := beh_ctrl_ptr_
  motorL_ptr          := motorL_ptr_
  motorR_ptr          := motorR_ptr_
  messages_ptr        := messages_ptr_

  long[ir_detect_state_ptr] := IR_NOTHING

  long[messages_ptr] := @msg_buf[0]

  msg_disabled := string("disabled          ")
  msg_nothing  := string("nothing           ")
  msg_L        := string("turning left      ")      
  msg_R        := string("turning right     ")      
  msg_rev      := string("reverse           ")
  msg_pursuit  := string("pursuit           ")
  msg_shoot    := string("shoot             ")
  msg_shoot1   := string("shoot 1           ")
  msg_shoot2   := string("shoot 2           ")
  msg_shoot3   := string("shoot 3           ")
  msg_front    := string("front             ") 
  msg_start    := string("start             ") 

  bytefill(@msg_buf, " ", BUFSIZE - 1) 
  msg_buf[BUFSIZE - 1] := 0

  long[messages_ptr] := msg_start

  clk.init(_xinfreq)
  clk.pausemsec(100) ' some time to settle
  cognew(attack_machine, @stackATK)

PRI attack_machine | atk_state, diff, str_ptr
  dira[CANNON1_PIN]   := OUT
  dira[CANNON2_PIN]   := OUT
  dira[CANNON3_S_PIN] := OUT

  diff := 0

  atk_state := ATK_NOTHING 

  bullets_left := 3


  repeat
    process_ir 
    case atk_state
      ATK_NOTHING:
        long[messages_ptr] := msg_nothing

        if long[ir_detect_ptr][IRFL_PIN] and long[ir_detect_ptr][IRFR_PIN]
          atk_state := ATK_FRONT 

        elseif long[ir_detect_ptr][IRFL_PIN] 
          atk_state := ATK_FRONT

        elseif long[ir_detect_ptr][IRFR_PIN] 
          atk_state := ATK_FRONT

        elseif long[ir_detect_ptr][IRR_PIN] 
          hard_right
          atk_state := ATK_RIGHT 

        elseif long[ir_detect_ptr][IRL_PIN]
          hard_left
          atk_state := ATK_LEFT  
      
        elseif long[ir_detect_ptr][IRBR_PIN]
          hard_right
          atk_state := ATK_RIGHT  

        elseif long[ir_detect_ptr][IRBL_PIN] 
          hard_left
          atk_state := ATK_LEFT  

        else
          disable
          atk_state := ATK_NOTHING   

      ATK_FRONT:
       
        long[messages_ptr] := msg_front
        diff := long[ir_detect_ctr_ptr][IRFR_PIN] - long[ir_detect_ctr_ptr][IRFL_PIN] 


        if long[ir_detect_ptr][IRFL_PIN] and long[ir_detect_ptr][IRFR_PIN]
          stop
          atk_state := ATK_SHOOT 

        elseif long[ir_detect_ptr][IRFL_PIN] 
          hard_left
          atk_state := ATK_FRONT

        elseif long[ir_detect_ptr][IRFR_PIN] 
          hard_right
          atk_state := ATK_FRONT

        elseif long[ir_detect_ptr][IRR_PIN] 
          hard_right
          atk_state := ATK_RIGHT 

        elseif long[ir_detect_ptr][IRL_PIN]
          hard_left
          atk_state := ATK_LEFT  
      
        elseif long[ir_detect_ptr][IRBR_PIN]
          hard_right
          atk_state := ATK_RIGHT  

        elseif long[ir_detect_ptr][IRBL_PIN] 
          hard_left
          atk_state := ATK_LEFT  

        else
          disable
          atk_state := ATK_NOTHING   

      ATK_LEFT:

        long[messages_ptr] := msg_L

        if long[ir_detect_ptr][IRFL_PIN] and long[ir_detect_ptr][IRFR_PIN]
          stop
          atk_state := ATK_FRONT 

        elseif long[ir_detect_ptr][IRFL_PIN] 
          soft_left
          atk_state := ATK_FRONT

        elseif long[ir_detect_ptr][IRFR_PIN] 
          stop
          atk_state := ATK_FRONT

        elseif long[ir_detect_ptr][IRR_PIN] 
          stop
          atk_state := ATK_RIGHT 

        elseif long[ir_detect_ptr][IRL_PIN]
          hard_left
          atk_state := ATK_LEFT  
      
        elseif long[ir_detect_ptr][IRBR_PIN]
          stop
          atk_state := ATK_RIGHT  

        elseif long[ir_detect_ptr][IRBL_PIN] 
          hard_left
          atk_state := ATK_LEFT  

        else
          disable
          atk_state := ATK_NOTHING   

      ATK_RIGHT:

        long[messages_ptr] := msg_R
        if long[ir_detect_ptr][IRFL_PIN] and long[ir_detect_ptr][IRFR_PIN]
          stop
          atk_state := ATK_FRONT 

        elseif long[ir_detect_ptr][IRFL_PIN] 
          stop
          atk_state := ATK_FRONT

        elseif long[ir_detect_ptr][IRFR_PIN] 
          soft_right
          atk_state := ATK_FRONT

        elseif long[ir_detect_ptr][IRR_PIN] 
          hard_left
          atk_state := ATK_RIGHT 

        elseif long[ir_detect_ptr][IRL_PIN]
          stop
          atk_state := ATK_LEFT  
      
        elseif long[ir_detect_ptr][IRBR_PIN]
          hard_right
          atk_state := ATK_RIGHT  

        elseif long[ir_detect_ptr][IRBL_PIN] 
          stop
          atk_state := ATK_LEFT  

        else
          disable
          atk_state := ATK_NOTHING   

      ATK_SHOOT:

        long[messages_ptr] := msg_shoot
        if long[ir_detect_ptr][IRFL_PIN] and long[ir_detect_ptr][IRFR_PIN]
          
          attack_accumulator := 0
          repeat 5
            if long[ir_detect_ptr][IRFL_PIN] and long[ir_detect_ptr][IRFR_PIN]
              attack_accumulator++
              clk.pausemsec(5)

          diff := long[ir_detect_ctr_ptr][IRFR_PIN] - long[ir_detect_ctr_ptr][IRFL_PIN] 
          if ||diff < 8 
            if bullets_left > 0
              shoot 
              atk_state := ATK_NOTHING 
            else
              forward(0)
              atk_state := ATK_SHOOT 

          elseif diff < 0
            soft_left
            atk_state := ATK_FRONT
          else
            soft_right 
            atk_state := ATK_FRONT

        elseif long[ir_detect_ptr][IRFL_PIN] 
          soft_left
          atk_state := ATK_FRONT

        elseif long[ir_detect_ptr][IRFR_PIN] 
          soft_right
          atk_state := ATK_FRONT

        elseif long[ir_detect_ptr][IRR_PIN] 
          soft_right
          atk_state := ATK_FRONT 

        elseif long[ir_detect_ptr][IRL_PIN]
          soft_left
          atk_state := ATK_FRONT  
      
        elseif long[ir_detect_ptr][IRBR_PIN]
          soft_right
          atk_state := ATK_RIGHT  

        elseif long[ir_detect_ptr][IRBL_PIN] 
          soft_left
          atk_state := ATK_LEFT  

        else
          stop
          atk_state := ATK_FRONT   

      {ATK_PURSUIT:
        if long[ir_detect_ptr][IRFL_PIN] and long[ir_detect_ptr][IRFR_PIN]
          atk_state := ATK_FRONT 

        elseif long[ir_detect_ptr][IRFL_PIN] 
          atk_state := ATK_FRONT

        elseif long[ir_detect_ptr][IRFR_PIN] 
          atk_state := ATK_FRONT

        elseif long[ir_detect_ptr][IRR_PIN] 
          atk_state := ATK_RIGHT 

        elseif long[ir_detect_ptr][IRL_PIN]
          atk_state := ATK_LEFT  
      
        elseif long[ir_detect_ptr][IRBR_PIN]
          atk_state := ATK_RIGHT  

        elseif long[ir_detect_ptr][IRBL_PIN] 
          atk_state := ATK_LEFT  

        else
          atk_state := ATK_NOTHING}


PRI process_ir | i
  repeat i from 0 to irdet#MAX_IR_SENSORS - 1
    long[ir_detect_ptr][i] := (long[ir_detect_ctr_ptr][i] > irdet#IR_PULSE_THRESH)

PRI shoot | str_ptr
  if bullets_left == 0

  case bullets_left
    0: 
      'chase
      long[messages_ptr] := msg_pursuit
      forward(0)
    1: 

      long[messages_ptr] := msg_shoot1
      if SERVO_ENABLED
        servos.Start ' servo.set BEFORE servos.start
      ' note: cannon servo is physically in place of cannon 3
      ' but we'sll shoot it 1st
        servos.set(CANNON3_S_PIN, CANNON_S_FINAL)
        clk.pausemsec(250)
        servos.set(CANNON3_S_PIN, 0)
        clk.pausemsec(SHOOT_WAIT_TIME)
      bullets_left--
    2:
      long[messages_ptr] := msg_shoot1
      outa[CANNON1_PIN] := 1
      clk.pausemsec(25)
      outa[CANNON1_PIN] := 0
      clk.pausemsec(SHOOT_WAIT_TIME)
      bullets_left--
    3:
      long[messages_ptr] := msg_shoot1
      outa[CANNON2_PIN] := 1
      clk.pausemsec(25)
      outa[CANNON2_PIN] := 0
      clk.pausemsec(SHOOT_WAIT_TIME)
      bullets_left--

PRI forward(bias)
  long[beh_ctrl_ptr] := 1
  long[motorL_ptr] := omot#SPEED
  long[motorR_ptr] := omot#SPEED

PRI soft_right
  long[beh_ctrl_ptr]:= 1
  long[motorL_ptr]  := 0
  long[motorR_ptr]  := omot#SPEED

PRI soft_left
  long[beh_ctrl_ptr]:= 1
  long[motorL_ptr]  := 0
  long[motorR_ptr]  := omot#SPEED

PRI hard_right
  long[beh_ctrl_ptr]:= 1
  long[motorL_ptr]  := omot#SPEED
  long[motorR_ptr]  := -omot#SPEED

PRI hard_left
  long[beh_ctrl_ptr] := 1
  long[motorL_ptr] := -omot#SPEED
  long[motorR_ptr] := omot#SPEED

PRI disable
  long[beh_ctrl_ptr] := 0
  long[motorL_ptr] := long[motorR_ptr] := 0
 
PRI stop
  long[beh_ctrl_ptr] := 1            
  long[motorL_ptr] := long[motorR_ptr] := 0

'00 06 80 80 01 00  n
'00 00 80 80 80 00
'00 00 00 70 70 00
'00 80 99 80 00 00 
'00 80 00 39 00 00
'
'n 15cm  
'n-30deg 
'n-45deg 
'n+30deg 
'n+45deg 
