CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  '------------- 
  SW_L_PIN      = 6
  SW_R_PIN      = 8

  DEBOUNCE_THRESH = 20  ' for switches

  ' Escape constants ====
  ESC_REVERSE_TIME= 1200 ' reverse time in ms, was 2000
  ESC_TURN_TIME   = 1000 ' turn time in ms
  IN              = 0

  SPEED_SAMPLING_INTERVAL = 20_000_000

  ODOM_R_PIN      = 12
  ODOM_L_PIN      = 13
  WHEEL_DEBOUNCE  = 20
  OUT             = 1

  MAX_TICKS_PER_S = 32

OBJ 
  clk : "clock"
  omot: "open_motor"

VAR
  long pressedL_ptr
  long pressedR_ptr
  long beh_ctrl_ptr
  long motorL_ptr
  long motorR_ptr
  long messages_ptr

  byte pinctrL, pinctrR 

  long msg_disabled
  long msg_R
  long msg_L
  long msg_rev

  long stackESC[80]

  ' wheel watcher vars
  long stateL, stateR, last_changeR, last_changeL
  'tick ctrs
  long wheelL_ctr, wheelR_ctr
  long last_speed_sampling
  long speedL, speedR
  long speedL_ptr, speedR_ptr

PUB start(pressedL_ptr_, pressedR_ptr_, speedL_ptr_, speedR_ptr_, beh_ctrl_ptr_, motorL_ptr_, motorR_ptr_, messages_ptr_)
  pressedL_ptr := pressedL_ptr_
  pressedR_ptr := pressedR_ptr_
  beh_ctrl_ptr := beh_ctrl_ptr_
  motorL_ptr   := motorL_ptr_
  motorR_ptr   := motorR_ptr_
  messages_ptr := messages_ptr_
  speedR_ptr   := speedR_ptr_
  speedL_ptr   := speedL_ptr_

  msg_disabled := string("disabled          ")
  msg_L        := string("turning left      ")      
  msg_R        := string("turning right     ")      
  msg_rev      := string("reverse           ")

  long[speedR_ptr] := 0
  long[speedL_ptr] := 0
  wheelR_ctr := 0
  wheelL_ctr := 0
  speedL     := 0
  speedR     := 0

  byte[pressedL_ptr] := byte[pressedR_ptr] := 0

  pinctrL := pinctrR := 0

  clk.init(_xinfreq)

  long[messages_ptr] := msg_disabled

  cognew(escape, @stackESC)

PRI switch_detect 
  if (ina[SW_L_PIN] == 1)      
    if (pinctrL < DEBOUNCE_THRESH)
      pinctrL++
    else
      byte[pressedL_ptr] := $FF
  else
    byte[pressedL_ptr] := 0
    pinctrL := 0

  '----------------
  if (ina[SW_R_PIN] == 1)
    if (pinctrR < DEBOUNCE_THRESH)
      pinctrR++
    else
      byte[pressedR_ptr] := $FF
  else
    byte[pressedR_ptr] := 0
    pinctrR := 0

PRI watch_wheels 
  if ina[ODOM_L_PIN] <> stateL      
    if (cnt - last_changeL) > WHEEL_DEBOUNCE
      last_changeL := cnt
      wheelL_ctr++
      stateL := ina[ODOM_L_PIN]
  else
    last_changeL #>= (cnt - WHEEL_DEBOUNCE)

  if ina[ODOM_R_PIN] <> stateR      
    if (cnt - last_changeR) > WHEEL_DEBOUNCE
      last_changeR := cnt
      wheelR_ctr++
      stateR := ina[ODOM_R_PIN]
  else
    last_changeR #>= (cnt - WHEEL_DEBOUNCE)
 
  if (cnt - last_speed_sampling) > SPEED_SAMPLING_INTERVAL   
    'long[speedL_ptr] :=  (4*100*wheelL_ctr)/MAX_TICKS_PER_S
    'long[speedR_ptr] :=  (4*100*wheelR_ctr)/MAX_TICKS_PER_S
    long[speedL_ptr] :=  4*wheelL_ctr
    long[speedR_ptr] :=  4*wheelR_ctr
    wheelL_ctr := wheelR_ctr := 0
    last_speed_sampling := cnt

PRI escape
  dira[SW_L_PIN] := IN 
  dira[SW_R_PIN] := IN 
  dira[ODOM_R_PIN] := IN
  dira[ODOM_L_PIN] := IN

  stateR := ina[ODOM_R_PIN]
  stateL := ina[ODOM_L_PIN] 

  last_changeR := last_changeL := cnt - WHEEL_DEBOUNCE
  last_speed_sampling := cnt 

  repeat

    switch_detect
    watch_wheels 

    if (byte[pressedL_ptr] == $FF)
      reverse
      hard_right
      disable

    elseif (byte[pressedR_ptr] == $FF)
      reverse
      hard_left
      disable

    else
      disable

    clk.pausemsec(5)

PRI reverse
  long[messages_ptr] := msg_rev 
  long[beh_ctrl_ptr] := 1            
  long[motorL_ptr] := long[motorR_ptr] := -omot#SPEED
  clk.pausemsec(ESC_REVERSE_TIME)
  short_stop

PRI hard_right
  long[messages_ptr] := msg_R
  long[beh_ctrl_ptr] := 1            
  long[motorL_ptr] := omot#SPEED
  long[motorR_ptr] := -omot#SPEED
  clk.pausemsec(ESC_TURN_TIME)            
  short_stop

PRI hard_left
  long[messages_ptr] := msg_L
  long[beh_ctrl_ptr] := 1            
  long[motorL_ptr] := -omot#SPEED
  long[motorR_ptr] := omot#SPEED
  clk.pausemsec(ESC_TURN_TIME)            
  short_stop

PRI short_stop
  long[beh_ctrl_ptr] := 1            
  long[motorL_ptr] := long[motorR_ptr] := 0
  ' clk.pausemsec(20)

PRI disable
  long[messages_ptr] := msg_disabled
  long[beh_ctrl_ptr] := 0
  long[motorL_ptr] := long[motorR_ptr] := 0
  
