CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  EN34_PIN      = 21
  IN4_PIN       = 20
  IN3_PIN       = 19
  EN12_PIN      = 18 ' beware of cables
  IN2_PIN       = 17
  IN1_PIN       = 16
  DELAY_PER_STEP= 75_000
  PWM_PERIOD    = 10000
  FWD           = 80
  MAX_POWER     = 80
  ' default
  SPEED         = 80

  MAX_TICKS_PER_S = 32

  OUT           = 1

OBJ
  pwmL : "pwmAsm"
  pwmR : "pwmAsm"

VAR
  ' signal sent to hbridge
  long powerL, powerR
  ' desired speed
  long targetR, targetL

PUB start
  dira[IN1_PIN] := OUT
  dira[IN2_PIN] := OUT
  dira[IN3_PIN] := OUT                                    
  dira[IN4_PIN] := OUT

  ' init motors
  pwmL.start(EN12_PIN)
  pwmR.start(EN34_PIN)
  pwmL.setPeriod(PWM_PERIOD)
  pwmR.setPeriod(PWM_PERIOD)

  powerL := 0
  powerR := 0
  targetL:= 0
  targetR:= 0

PUB set_speed(targetL_, targetR_)
  ' targets are in (-100,100) range
  targetL := targetL_ 
  targetR := targetR_ 

  ' do ramping
  repeat until targetR == powerR and targetL == powerL
    if targetR > powerR
      powerR++          
    elseif targetR < powerR
      powerR--
    if targetL > powerL
      powerL++          
    elseif targetL < powerL
      powerL--
    set_hbridge(powerL, powerR)
    waitcnt(cnt + DELAY_PER_STEP)

PRI set_hbridge(sgn_left, sgn_right) | in1, in2, in3, in4 
  ' input: signed speed/power
  ' but L298 uses direction + pwm
  ' in[1-4] determines direction, pwm determines power
  if sgn_right > 0
    in1 := 1
    in2 := 0
  elseif sgn_right < 0
    in1 := 0 
    in2 := 1 
  else
    in1 := in2 := 0

  if sgn_left > 0
    in3 := 1
    in4 := 0
  elseif sgn_left < 0
    in3 := 0 
    in4 := 1 
  else
    in3 := in4 := 0

  ' keep absolute value of input for pwm
  ||sgn_right
  ||sgn_left 

  outa[IN1_PIN] := in1        
  outa[IN2_PIN] := in2
  outa[IN3_PIN] := in3
  outa[IN4_PIN] := in4
  pwmL.setDuty(sgn_left)
  pwmR.setDuty(sgn_right)
