CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  MOTORS_ENABLED = 0
  IR_PULSE_COUNT_THRESH = 40
  IR_BEH_ENABLED = 1
  NAV_BEH_ENABLED = 1
  HUNT_BEH_ENABLED = 1
  ESC_BEH_ENABLED = 1
  RAND_CHASE = 20
  '-------------------------------------------------------------------------
  'time
  ms = 80_000
  sec = 80_000_000
  long_interval = 60_000_000
  short_interval = 17_000_000
  '-------------------------------------------------------------------------
  MAX_BEH = 3
  #0, ESCAPE_BEH, HUNT_BEH, NAV_BEH   
  '-------------------------------------------------------------------------
  ' Pin defs
  GLED_PIN = 19
  RLED_PIN = 23
  '-------------
  PING_L_PIN = 7
  PING_M_PIN = 8
  PING_R_PIN = 9
  '------------- 
  SW_L_PIN = 13
  SW_R_PIN = 15
  '-------------
  IRBL_PIN = 1
  IRL_PIN =  2
  IRFL_PIN = 3  
  IRFR_PIN = 4
  IRR_PIN =  5
  IRBR_PIN = 6                
  '-------------
  CANNON1_PIN = 26
  CANNON2_PIN = 25
  CANNON3_PIN = 27
  '-------------
  CANNON3_S_PIN = 27 ' in place of 3, a servo
  ' motor control
  DELAY_PER_STEP = 3
  PWM_FREQ = 10000
  EN34_PIN = 21
  IN4_PIN = 22
  IN3_PIN = 20
  EN12_PIN = 18 ' beware of cables
  IN2_PIN = 17
  IN1_PIN = 16
  '------------------
  OUT = 1
  IN = 0
  MIN_CNT = 400 ' minimum value for waitcnt
  '------------------
  'servo speeds
  FWD = 70
  STOP = 0
  ' hard turn params
  #1, SLOW
  #1, LEFT, RIGHT
  ' cannon servo parms
  CANNON_S_INITIAL = 1982
  CANNON_S_FINAL = 1200
  '-------------------------------------------------------------------------
  ' misc thresholds
  DEBOUNCE_THRESH = 20  ' for switches
  IR_FREQ = 466
  MAX_IR_SENSORS = 6
  MAX_IR_CTR = 2
  IR_PULSE_THRESH = 10
  IR_MEASURE_TIME = 50
  ' location of ir info
  #0, IRBL, IRL, IRFL, IRFR, IRR, IRBR
  #1, IR_NOTHING, IR_LEFT, IR_LEFT_FRONT, IR_RIGHT, IR_RIGHT_FRONT, IR_LOCK
  '-------------------------------------------------------------------------
  ' Escape constants
  ESC_REVERSE_TIME = 1200 ' reverse time in ms, was 2000
  ESC_TURN_TIME = 600 ' turn time in ms
  ' Nav constants
  NAV_HARD_TURN_TIME = 200 'was 500
  NAV_HARD_TURN_TIME_LONG = 400
  NAV_HARD_TURN_TIME_VERY_LONG = 600
  TOO_CLOSE_THRESH = 20 'sonar detected centimeters to back up, in 
  TOO_TOO_CLOSE_THRESH = 11
  HUNT_TURN_TIME = 40 'was 200
  HUNT_TURN_TIME_SHORT = 10
  SHOOT_THRESH = 3
  HUNT_FWD_TIME = 300
  MENEO_THRESH = 8
  MENEO_TIME = 5
  SHOOT_WAIT_TIME = 500
  CRUISE_TIME = 80
  '---
  #1,HUNT_FRONT, HUNT_LEFT, HUNT_RIGHT, HUNT_SHOOT, HUNT_CHASE, HUNT_NOTHING 

OBJ 
  timing : "Timing" ' for busy delays
  pingL : "Ping"
  pingM : "Ping"
  pingR : "Ping"
  servos: "Servo32" ' servos contol, 1 cog
  bs2 : "BS2_Functions"
  lcd : "Debug_Lcd"
  pwmL : "pwmAsm"
  pwmR : "pwmAsm"

VAR
  ' cog stacks  
  long stackESC[80]
  long stackHUNT[120]                         
  long stackNAV[120]
  long stackIR[80]
  ' behavior related
  byte beh_ctrl[MAX_BEH]
  long motor_left[MAX_BEH]
  long motor_right[MAX_BEH]
  long motor_IN1[MAX_BEH]
  long motor_IN2[MAX_BEH]
  long motor_IN3[MAX_BEH]
  long motor_IN4[MAX_BEH]
  long messages[MAX_BEH]
  ' ir detection 
  long freq[MAX_IR_SENSORS]
  byte ir_pins[MAX_IR_SENSORS]
  byte ir_detect[MAX_IR_SENSORS]
  long ir_detect_ctr[MAX_IR_SENSORS]
  ' sonar ranges
  long rangeL, rangeM, rangeR
  long rangeL0, rangeM0, rangeR0
  ' whiskers
  byte pressedL,pressedR
  byte pinctrL, pinctrR
  ' hunting
  byte hunt_ctr, meneo, ir_detect_state
 

PUB Main | c,range,blinky,i
  'debug(cannon_s_debug) ' debug cannon servo before servo setup
  timing.pause1ms(1000)
  '-------- Startup routines
  startup
  repeat i from 0 to 3
    lcd.clrln(i)
  'cannon_debug  
  'debugIR_draw_lcd
  '-------- cog launches
  if IR_BEH_ENABLED
    cognew(ir_detectornew, @stackIR)
  if ESC_BEH_ENABLED
    cognew(escape, @stackESC)
  if HUNT_BEH_ENABLED
    cognew(hunt_ir_2, @stackHUNT)
    'cognew(hunt_ir, @stackHUNT)
  if NAV_BEH_ENABLED
    cognew(navigate2, @stackNAV)
    'cognew(navigate, @stackNAV)
  '-------------------
  repeat
    lcd_draw
    repeat i from 0 to (MAX_BEH-1)
      if (beh_ctrl[i] == 1)
        lcd.gotoxy(0,2)
        lcd.str(string("ML: "))
        lcd.decx(motor_IN1[i],1)
        lcd.decx(motor_IN2[i],1)
        lcd.putc(",")                  
        lcd.decx(motor_left[i],2)
        lcd.str(string("|MR: "))        
        lcd.decx(motor_IN1[i],1)
        lcd.decx(motor_IN2[i],1)
        lcd.putc(",")
        lcd.decx(motor_right[i],2)
        lcd.gotoxy(0,3)
        lcd.str(messages[i])
        if (MOTORS_ENABLED)                                     
          outa[IN1_PIN] := motor_IN1[i]        
          outa[IN2_PIN] := motor_IN2[i]
          outa[IN3_PIN] := motor_IN3[i]
          outa[IN4_PIN] := motor_IN4[i]
          pwmL.setDuty(motor_left[i])
          pwmR.setDuty(motor_right[i])
        quit  ' like break in c
      timing.pause1ms(5)      
    !outa[GLED_PIN]      

PRI startup | okay
  pinctrL := pinctrR := 0
  pressedL := pressedR := 0
  hunt_ctr := 0
  ir_pins[IRBL] := IRBL_PIN
  ir_pins[IRL] := IRL_PIN
  ir_pins[IRFL] := IRFL_PIN
  ir_pins[IRFR] := IRFR_PIN  
  ir_pins[IRR] := IRR_PIN
  ir_pins[IRBR] := IRBR_PIN
  '-------- pin directions
  dira[PING_L_PIN] := IN
  dira[PING_R_PIN] := IN
  dira[PING_M_PIN] := IN
  dira[GLED_PIN] := OUT
  dira[RLED_PIN] := OUT
  dira[CANNON1_PIN] := OUT
  dira[CANNON2_PIN] := OUT
  dira[CANNON3_PIN] := OUT
  dira[CANNON3_S_PIN] := OUT
  dira[SW_R_PIN] := IN
  dira[SW_L_PIN] := IN
  dira[IN1_PIN] := dira[IN2_PIN] := dira[IN3_PIN] := dira[IN4_PIN] := OUT
  'dira[EN12_PIN] := dira[EN34_PIN] := OUT
  'outa[EN12_PIN] := OUT
  'outa[EN34_PIN] := OUT
  pwmL.start(EN12_PIN)
  pwmR.start(EN34_PIN)
  pwmL.setPeriod(30000)
  pwmR.setPeriod(30000)    
  '-------- object init
  bs2.start(31,30)  
  lcd.start(0,19200,4)
  lcd.backlight(true)
  lcd.cls
  lcd.cls
  servos.Set(CANNON3_S_PIN, CANNON_S_INITIAL)  
  servos.Start ' servo.set BEFORE servos.start
  timing.pause1ms(100) ' some time to settle

PRI lcd_draw | i
  lcd.home      ' 0,0
  lcd.str(string("S:"))
  lcd.decx(rangeL,3)
  lcd.putc(" ")
  lcd.decx(rangeM,3)
  lcd.putc(" ")
  lcd.decx(rangeR,3)
  lcd.str(string(",L"))
  lcd.decx(pressedL,1)
  lcd.str(string(" R"))
  lcd.decx(pressedR,1)
  lcd.gotoxy(0,1)
  'lcd.str(string("IR:"))
  'repeat i from 0 to 5
  '  lcd.decx(ir_detect[i],1)
  'lcd.gotoxy(0,2)
    case ir_detect_state
      IR_NOTHING:
        lcd.str(string("IR_NOTHING         "))
      IR_LEFT:
        lcd.str(string("IR_LEFT            "))
      IR_RIGHT:
        lcd.str(string("IR_RIGHT           "))
      IR_LEFT_FRONT:
        lcd.str(string("IR_LEFT_FRONT      "))
      IR_RIGHT_FRONT:
        lcd.str(string("IR_RIGHT_FRONT     "))
    
PRI debugIR_draw_lcd | i
  lcd.cls
  repeat i from 0 to MAX_IR_SENSORS - 1
    dira[ir_pins[i]] := IN
  repeat  
    lcd.home
    repeat i from 0 to MAX_IR_SENSORS - 1
      lcd.decx(bs2.count(ir_pins[i],100),2)      
      lcd.putc(" ")        

PRI switch_detect 
  dira[SW_L_PIN] := dira[SW_R_PIN] := IN 
  if (ina[SW_L_PIN] == 1)      
    if (pinctrL < DEBOUNCE_THRESH)
      pinctrL++
    else
      pressedL := 1
  else
    pressedL := 0
    pinctrL := 0
  '----------------
  if (ina[SW_R_PIN] == 1)
    if (pinctrR < DEBOUNCE_THRESH)
      pinctrR++
    else
      pressedR := 1
  else
    pressedR := 0
    pinctrR := 0
  '-----------------

PRI escape
  ' escape behavior, depends on whisker state
  repeat
    switch_detect
    if (pressedL)
      messages[ESCAPE_BEH] := string("ESC: pressed L      ")      
      beh_ctrl[ESCAPE_BEH] := 1            
      reverse(ESCAPE_BEH)
      timing.pause1ms(ESC_REVERSE_TIME)
      stop_motors(ESCAPE_BEH)
      timing.pause1ms(20)
      messages[ESCAPE_BEH] := string("ESC: turning right  ")  
      hard_turn(RIGHT, ESCAPE_BEH)
      timing.pause1ms(ESC_TURN_TIME)            
      messages[ESCAPE_BEH] := string("ESC: stop           ")
      stop_motors(ESCAPE_BEH)
      timing.pause1ms(20)
      beh_ctrl[ESCAPE_BEH] := 0      
    elseif (pressedR)
      messages[ESCAPE_BEH] := string("ESC: pressed R      ")
      beh_ctrl[ESCAPE_BEH] := 1
      reverse(ESCAPE_BEH)
      timing.pause1ms(ESC_REVERSE_TIME)
      stop_motors(ESCAPE_BEH)
      timing.pause1ms(20)   
      messages[ESCAPE_BEH] := string("ESC: turning left   ")
      hard_turn(LEFT, ESCAPE_BEH)
      timing.pause1ms(ESC_TURN_TIME)
      messages[ESCAPE_BEH] := string("ESC: stop           ")
      stop_motors(ESCAPE_BEH)
      timing.pause1ms(50)
      beh_ctrl[ESCAPE_BEH] := 0    
    else
      beh_ctrl[ESCAPE_BEH] := 0
      stop_motors(ESCAPE_BEH)
    timing.pause1ms(5)

PRI navigate
  repeat    
    rangeL += pingL.centimeters(PING_L_PIN)
    rangeL /=2
    timing.pause1ms(30)
    rangeR += pingR.centimeters(PING_R_PIN)
    rangeR /=2
    timing.pause1ms(30)
    rangeM += pingM.centimeters(PING_M_PIN)
    rangeM /=2
    timing.pause1ms(30)
    if (rangeM < TOO_TOO_CLOSE_THRESH) or (rangeR < TOO_TOO_CLOSE_THRESH) or (rangeR < TOO_TOO_CLOSE_THRESH)
      beh_ctrl[NAV_BEH] := 1 
      messages[NAV_BEH] := string("NAV: too close front")
      stop_motors(NAV_BEH)
      reverse(NAV_BEH)
      timing.pause1ms(550)
      stop_motors(NAV_BEH)
      timing.pause1ms(20)
      hard_turn(RIGHT, NAV_BEH)
      timing.pause1ms(900)      
    elseif (rangeL < TOO_CLOSE_THRESH) and (rangeR < TOO_CLOSE_THRESH) and (rangeM < TOO_CLOSE_THRESH)
      messages[NAV_BEH] := string("NAV: hard right      ")
      beh_ctrl[NAV_BEH] := 1
      stop_motors(NAV_BEH)
      reverse(NAV_BEH)
      timing.pause1ms(550)
      stop_motors(NAV_BEH)
      hard_turn(RIGHT, NAV_BEH)
      timing.pause1ms(NAV_HARD_TURN_TIME_VERY_LONG)
    elseif (rangeL < TOO_CLOSE_THRESH) and (rangeM < TOO_CLOSE_THRESH)
      messages[NAV_BEH] := string("NAV: hard right      ")
      hard_turn(RIGHT, NAV_BEH)
      timing.pause1ms(NAV_HARD_TURN_TIME_LONG)
      stop_motors(NAV_BEH)
      timing.pause1ms(10)
    elseif (rangeR < TOO_CLOSE_THRESH) and (rangeM < TOO_CLOSE_THRESH)
      messages[NAV_BEH] := string("NAV: hard left       ")
      hard_turn(RIGHT, NAV_BEH)
      timing.pause1ms(NAV_HARD_TURN_TIME_LONG)
      stop_motors(NAV_BEH)
      timing.pause1ms(10)                  
    elseif (rangeM < TOO_CLOSE_THRESH)
      messages[NAV_BEH] := string("NAV: Too close m     ")
      if (rangeR > rangeL)
        ' turn right
        ' hard_turn(right_or_left, speed, behavior_num)
        messages[NAV_BEH]:=string("NAV: hard right      ")
        hard_turn(RIGHT, NAV_BEH)
        timing.pause1ms(NAV_HARD_TURN_TIME_LONG)
        stop_motors(NAV_BEH)           
      else
        ' turn left
        messages[NAV_BEH]:=string("NAV: hard left       ")
        hard_turn(LEFT, NAV_BEH)
        timing.pause1ms(NAV_HARD_TURN_TIME_LONG)
        stop_motors(NAV_BEH)                  
    elseif (rangeL < TOO_CLOSE_THRESH)
      messages[NAV_BEH] := string("NAV: hard right      ")
      hard_turn(RIGHT, NAV_BEH)
      timing.pause1ms(NAV_HARD_TURN_TIME)
      stop_motors(NAV_BEH)
    elseif (rangeR < TOO_CLOSE_THRESH)
      messages[NAV_BEH] := string("NAV: hard left       ")
      hard_turn(LEFT, NAV_BEH)
      timing.pause1ms(NAV_HARD_TURN_TIME)
      stop_motors(NAV_BEH)
    else                   
      'motor_left[NAV_BEH] := FWD
      'motor_right[NAV_BEH] := FWD
      forward(NAV_BEH)
      timing.pause1ms(CRUISE_TIME)
    messages[NAV_BEH] :=   string("NAV: cruise          ")
    beh_ctrl[NAV_BEH] := 1

PRI navigate2 | diff, absdiff
  repeat    
    rangeL += pingL.centimeters(PING_L_PIN)
    rangeL /=2
    timing.pause1ms(30)
    rangeR += pingR.centimeters(PING_R_PIN)
    rangeR /=2
    timing.pause1ms(30)
    rangeM += pingM.centimeters(PING_M_PIN)
    rangeM /=2
    timing.pause1ms(30)
    if (rangeM < TOO_TOO_CLOSE_THRESH) or (rangeR < TOO_TOO_CLOSE_THRESH) or (rangeR < TOO_TOO_CLOSE_THRESH)
      beh_ctrl[NAV_BEH] := 1 
      messages[NAV_BEH] := string("NAV: too2 close front")
      stop_motors(NAV_BEH)
      reverse(NAV_BEH)
      timing.pause1ms(550)
      stop_motors(NAV_BEH)
      timing.pause1ms(20)
      hard_turn(RIGHT, NAV_BEH)
      timing.pause1ms(900)
    elseif (rangeM < TOO_CLOSE_THRESH)
      messages[NAV_BEH] := string("NAV: too close front ")
      if (rangeR > rangeL)
        messages[NAV_BEH]:=string("NAV: hard right      ")
        hard_turn(RIGHT, NAV_BEH)
        timing.pause1ms(NAV_HARD_TURN_TIME_LONG)
        stop_motors(NAV_BEH)           
      else
        messages[NAV_BEH]:=string("NAV: hard left       ")
        hard_turn(LEFT, NAV_BEH)
        timing.pause1ms(NAV_HARD_TURN_TIME_LONG)
        stop_motors(NAV_BEH)
    elseif (rangeL < TOO_CLOSE_THRESH) 
      messages[NAV_BEH] := string("NAV: hard right      ")
      hard_turn(RIGHT, NAV_BEH)
      timing.pause1ms(NAV_HARD_TURN_TIME_LONG)
      stop_motors(NAV_BEH)
      timing.pause1ms(10)     
    elseif (rangeR < TOO_CLOSE_THRESH) 
      messages[NAV_BEH] := string("NAV: hard left       ")
      hard_turn(LEFT, NAV_BEH)
      timing.pause1ms(NAV_HARD_TURN_TIME_LONG)
      stop_motors(NAV_BEH)
      timing.pause1ms(10) 
    else
      messages[NAV_BEH] := string("NAV: cruise          ")      
      diff := rangeL - rangeR ' max should be around 300
      forward(NAV_BEH) ' both on FWD speed
      absdiff := ||diff
      absdiff <#= 30 ' absdiff = min(absdiff,30)
      if absdiff > 20        
        if (diff > 0) ' L > R -> more speed to right motor
          messages[NAV_BEH] := string("NAV: cruise left     ")
          motor_left[NAV_BEH] := (FWD - absdiff) #> 50 
          motor_right[NAV_BEH] := (FWD + absdiff) <# 99
        else
          messages[NAV_BEH] := string("NAV: cruise right    ")         
          motor_left[NAV_BEH] := (FWD + absdiff) <# 99
          motor_right[NAV_BEH] := (FWD - absdiff) #> 50                                            
      timing.pause1ms(CRUISE_TIME)
    beh_ctrl[NAV_BEH] := 1            
  
PRI ir_detectornew | i, diff
  repeat i from 0 to MAX_IR_SENSORS - 1
    dira[ir_pins[i]] := IN
  repeat
    repeat i from 0 to MAX_IR_SENSORS-1
      ir_detect_ctr[i] := bs2.count(ir_pins[i],100) ' count pulses for 100 ms
    repeat i from 0 to MAX_IR_SENSORS-1
      if ir_detect_ctr[i] > IR_PULSE_COUNT_THRESH
        ir_detect[i] := 1
      else
        ir_detect[i] := 0
    if (ir_detect[IRFL] and ir_detect[IRFR])
      ir_detect_state := IR_LOCK
    elseif (ir_detect[IRFL])
      ir_detect_state := IR_LEFT_FRONT
    elseif (ir_detect[IRFR])    
      ir_detect_state := IR_RIGHT_FRONT
    elseif (ir_detect[IRL])
      ir_detect_state := IR_LEFT
    elseif (ir_detect[IRR])
      ir_detect_state := IR_RIGHT
    elseif (ir_detect[IRBR])
      ir_detect_state := IR_RIGHT
    elseif (ir_detect[IRBL])
      ir_detect_state := IR_LEFT
    else
      ir_detect_state := IR_NOTHING                                  


PRI hunt_ir_2 | cannon_num, diff, absdiff, chase_ctr
  dira[CANNON1_PIN] := OUT
  dira[CANNON2_PIN] := OUT
  dira[CANNON3_PIN] := OUT
  cannon_num := 1 ' cannon to fire
  chase_ctr := 0
  repeat
    case ir_detect_state       
      IR_NOTHING:
        hunt_ctr := 0
        beh_ctrl[HUNT_BEH] := 0
        messages[HUNT_BEH] := string("HUN: nothing        ")
        
      IR_LEFT:
        hunt_ctr := 0
        beh_ctrl[HUNT_BEH] := 1
        messages[HUNT_BEH] := string("HUN: left           ")
        hard_turn(LEFT, HUNT_BEH)
        timing.pause1ms(HUNT_TURN_TIME_SHORT)
        stop_motors(HUNT_BEH)
        timing.pause1ms(5)
        
      IR_RIGHT:
        hunt_ctr := 0
        beh_ctrl[HUNT_BEH] := 1
        messages[HUNT_BEH] := string("HUN: right          ")
        hard_turn(RIGHT, HUNT_BEH)
        timing.pause1ms(HUNT_TURN_TIME_SHORT)
        stop_motors(HUNT_BEH)
        timing.pause1ms(5)

      IR_LEFT_FRONT:
        hunt_ctr := 0
        beh_ctrl[HUNT_BEH] := 1
        messages[HUNT_BEH] := string("HUN: left front     ")
        hard_turn(LEFT, HUNT_BEH)
        forward(HUNT_BEH)
        timing.pause1ms(250)
        timing.pause1ms(HUNT_TURN_TIME_SHORT)
        stop_motors(HUNT_BEH)
        timing.pause1ms(5)
        
      IR_RIGHT_FRONT:
        hunt_ctr := 0
        beh_ctrl[HUNT_BEH] := 1
        messages[HUNT_BEH] := string("HUN: right front    ")
        forward(HUNT_BEH)
        forward(HUNT_BEH)
        timing.pause1ms(250)
        hard_turn(RIGHT, HUNT_BEH)
        timing.pause1ms(HUNT_TURN_TIME_SHORT)
        stop_motors(HUNT_BEH)
        timing.pause1ms(5)
        
      IR_LOCK:
        beh_ctrl[HUNT_BEH] := 1
        messages[HUNT_BEH] := string("HUN: lock           ")
        timing.pause1ms(50)          
        if (hunt_ctr < SHOOT_THRESH)
          hunt_ctr++              
        else
           'shoot
          beh_ctrl[HUNT_BEH] := 1
          timing.pause1ms(30)
          if (cannon_num < 4)
            messages[HUNT_BEH] := string("HUN: shoot      ")
            stop_motors(HUNT_BEH)
            timing.pause1ms(SHOOT_WAIT_TIME)            
            shoot_cannon(cannon_num)
            cannon_num++
          else
          if (rangeM < TOO_TOO_CLOSE_THRESH) or (rangeR < TOO_TOO_CLOSE_THRESH) or (rangeR < TOO_TOO_CLOSE_THRESH)
            {
            chase_ctr++
            chase_ctr //= RAND_CHASE
            if chase_ctr == 0
              reverse(HUNT_BEH)
              timing.pause1ms(400)
              hard_turn(LEFT, HUNT_BEH)
              timing.pause1ms(200)
              stop_motors(HUNT_BEH)
              timing.pause1ms(10)
            
            chase_ctr++
            chase_ctr //= RAND_CHASE
            if chase_ctr==0
              beh_ctrl[HUNT_BEH] := 1 
              messages[HUNT_BEH] := string("HUN: too close stop ")
              stop_motors(HUNT_BEH)
              timing.pause1ms(10)
              messages[HUNT_BEH] := string("HUN: too close rev  ")
              reverse(HUNT_BEH)
              timing.pause1ms(500)
              stop_motors(HUNT_BEH)
              timing.pause1ms(20)
              messages[HUNT_BEH] := string("HUN: too close turn ")
              hard_turn(RIGHT, HUNT_BEH)
              timing.pause1ms(300)
            }                                              
            forward(HUNT_BEH)
            timing.pause1ms(200)
            messages[HUNT_BEH] := string("HUN: chase      ")
            beh_ctrl[HUNT_BEH] := 1
            diff := ir_detect_ctr[IRFL] - ir_detect_ctr[IRFR] ' around 0  - 30            
            absdiff := ||diff
            absdiff <#= 30
            if absdiff > 5 
              if (diff > 0) ' L > R -> more speed to right motor
                messages[HUNT_BEH] := string("HUN: chase left     ")
                motor_left[HUNT_BEH] := (FWD - absdiff) #> 50 
                motor_right[HUNT_BEH] := (FWD + absdiff) <# 99
              else
                messages[HUNT_BEH] := string("HUN: chase right    ")
                motor_left[HUNT_BEH] := (FWD + absdiff) <# 99
                motor_right[HUNT_BEH] := (FWD - absdiff) #> 50
            else
              forward(HUNT_BEH)
            timing.pause1ms(HUNT_FWD_TIME)
          hunt_ctr := 0 ' reset hunt_ctr        

PRI shoot_cannon(num)
  dira[CANNON1_PIN]   := OUT
  dira[CANNON2_PIN]   := OUT
  dira[CANNON3_PIN]   := OUT
  dira[CANNON3_S_PIN] := OUT
  case num
    ' note: cannon servo is physically in place of cannon 3
    ' but we'sll shoot it 1st
    1: ' shoot servo 1st
      servos.set(CANNON3_S_PIN, CANNON_S_FINAL)
      timing.pause1ms(250)
      ' done
    2:
      outa[CANNON1_PIN] := 1
      timing.pause1ms(25)
      outa[CANNON1_PIN] := 0
    3:
      outa[CANNON2_PIN] := 1
      timing.pause1ms(25)
      outa[CANNON2_PIN] := 0
            
PRI cannon_debug | i
  repeat i from 1 to 3
    shoot_cannon(i)
    timing.pause1ms(500)
        
